/*====================================================================
		rdatetime.c

		rFunc InterBase UDF library.
		Date&Time functions.

		Copyright 1998-2001 Polaris Software
		http://polesoft.da.ru
		mailto: rFunc@mail.ru

		Version 2.0.0.3
====================================================================== */

#include <time.h>
#include <stdlib.h>
#include <ibase.h>
#include "rfunc.h"
#include "rdatetime.h"

#ifndef IB_6X

long EXPORT fn_year(ARG(ISC_QUAD*, d))
ARGLIST(ISC_QUAD *d)
{
	struct tm tm1;
	isc_decode_date(d, &tm1);
	return (tm1.tm_year + IB_START_YEAR);
}

long EXPORT fn_month(ARG(ISC_QUAD*, d))
ARGLIST(ISC_QUAD *d)
{
	struct tm tm1;
	isc_decode_date(d, &tm1);
	return (tm1.tm_mon + 1);
}

long EXPORT fn_day(ARG(ISC_QUAD*, d))
ARGLIST(ISC_QUAD *d)
{
	struct tm tm1;
	isc_decode_date(d, &tm1);
	return tm1.tm_mday;
}

long EXPORT fn_yearday(ARG(ISC_QUAD*, d))
ARGLIST(ISC_QUAD *d)
{
	struct tm tm1;
	isc_decode_date(d, &tm1);
	return tm1.tm_yday;
}

long EXPORT fn_weekday(ARG(ISC_QUAD*, d))
ARGLIST(ISC_QUAD *d)
{
	struct tm tm1;
	isc_decode_date(d, &tm1);
	return tm1.tm_wday;
}

long EXPORT fn_hour(ARG(ISC_QUAD*, d))
ARGLIST(ISC_QUAD *d)
{
	struct tm tm1;
	isc_decode_date(d, &tm1);
	return tm1.tm_hour;
}

long EXPORT fn_minute(ARG(ISC_QUAD*, d))
ARGLIST(ISC_QUAD *d)
{
	struct tm tm1;
	isc_decode_date(d, &tm1);
	return tm1.tm_min;
}

long EXPORT fn_second(ARG(ISC_QUAD*, d))
ARGLIST(ISC_QUAD *d)
{
	struct tm tm1;
	isc_decode_date(d, &tm1);
	return tm1.tm_sec;
}

#endif

long EXPORT fn_quarter(ARG(ISC_QUAD*, d))
ARGLIST(ISC_QUAD *d)
{
	struct tm tm1;
	div_t x;

	isc_decode_date(d, &tm1);
	x = div(tm1.tm_mon, 3);
	return x.quot+1;
}

long	EXPORT fn_isleapyear(ARG(long, *ayear))
ARGLIST(long *ayear)
{
	return (long) isleap(*ayear);
}

char* EXPORT fn_datetostr(ARG(ISC_QUAD*, d), ARG(char*, fmt))
ARGLIST(ISC_QUAD *d)
ARGLIST(char *fmt)
{
	char *buffer = (char *) MALLOC (shortlen);
	struct tm tm1;
	isc_decode_date(d, &tm1);
	return (strftime(buffer, shortlen - 1, fmt, &tm1)) ? buffer : "";
}

ISC_QUAD* EXPORT fn_encodedate(ARG(long*, d), ARG(long*, m), ARG(long*, y))
ARGLIST(long *d)
ARGLIST(long *m)
ARGLIST(long *y)
{
	ISC_QUAD *bufquad = (ISC_QUAD *) MALLOC (sizeof(ISC_QUAD));
	struct tm tm1;
	tm1.tm_sec = 0;
	tm1.tm_min = 0;
	tm1.tm_hour = 0;
	tm1.tm_year = *y - IB_START_YEAR;
	tm1.tm_mon = *m - 1;
	tm1.tm_mday = *d;
	isc_encode_date(&tm1, bufquad);
	return bufquad;
}

ISC_QUAD* EXPORT fn_encodedatetime(ARG(long*, d), ARG(long*, m), ARG(long*, y), ARG(long*, hh), ARG(long*, mm), ARG(long*, ss))
ARGLIST(long *d)
ARGLIST(long *m)
ARGLIST(long *y)
ARGLIST(long *hh)
ARGLIST(long *mm)
ARGLIST(long *ss)
{
	ISC_QUAD *bufquad = (ISC_QUAD *) MALLOC (sizeof(ISC_QUAD));
	struct tm tm1;
	tm1.tm_sec = *ss;
	tm1.tm_min = *mm;
	tm1.tm_hour = *hh;
	tm1.tm_year = *y - IB_START_YEAR;
	tm1.tm_mon = *m - 1;
	tm1.tm_mday = *d;
	isc_encode_date(&tm1, bufquad);
	return bufquad;
}

ISC_QUAD* EXPORT fn_extractdate(ARG(ISC_QUAD*, d))
ARGLIST(ISC_QUAD *d)
{
	ISC_QUAD *bufquad = (ISC_QUAD *) MALLOC (sizeof(ISC_QUAD));

	*bufquad = *d;
	bufquad->gds_quad_low = 0;
	return bufquad;
}

ISC_QUAD*	EXPORT fn_extracttime(ARG(ISC_QUAD*, d))
ARGLIST(ISC_QUAD *d)
{
	ISC_QUAD *bufquad = (ISC_QUAD *) MALLOC (sizeof(ISC_QUAD));

	*bufquad = *d;
	bufquad->gds_quad_high = 0;
	return bufquad;
}

long	EXPORT fn_dow(ARG(ISC_QUAD*, d))
ARGLIST(ISC_QUAD *d)
{
	struct tm tm1;
	isc_decode_date(d, &tm1);
	return tm1.tm_wday==0 ? 7 : tm1.tm_wday;
}

long	EXPORT fn_daypermonth(ARG(long*, m), ARG(long*, y))
ARGLIST(long *m)
ARGLIST(long *y)
{
	short daysinmonth[12]={31, 28, 31, 30, 31, 30, 31, 31, 30, 31, 30, 31};
	if ((*m < 1 ) || (*m > 12)) return 0;
	return ((*m==2) && isleap(*y))
				 ? daysinmonth[*m-1]+1 : daysinmonth[*m-1] ;
}

ISC_QUAD*	EXPORT fn_firstdaymonth(ARG(ISC_QUAD*, d))
ARGLIST(ISC_QUAD* d)
{
	ISC_QUAD *bufquad = (ISC_QUAD *) MALLOC (sizeof(ISC_QUAD));
	struct tm tm1;

	isc_decode_date(d, &tm1);
	tm1.tm_hour = 0;
	tm1.tm_min = 0;
	tm1.tm_sec = 0;
	tm1.tm_mday = 1;
	isc_encode_date(&tm1, bufquad);

	return bufquad;
}

ISC_QUAD*	EXPORT fn_lastdaymonth(ARG(ISC_QUAD*, d))
ARGLIST(ISC_QUAD* d)
{
	ISC_QUAD *bufquad = (ISC_QUAD *) MALLOC (sizeof(ISC_QUAD));
	struct tm tm1;

	isc_decode_date(d, &tm1);
	tm1.tm_hour = 0;
	tm1.tm_min = 0;
	tm1.tm_sec = 0;
	tm1.tm_mon++;
	tm1.tm_mday = 0;
	isc_encode_date(&tm1, bufquad);

	return bufquad;
}

long	EXPORT fn_daysbetween(ARG(ISC_QUAD*, bdate), ARG(ISC_QUAD*, edate))
ARGLIST(ISC_QUAD* bdate)
ARGLIST(ISC_QUAD* edate)
{
	return edate->gds_quad_high - bdate->gds_quad_high;
}

ISC_QUAD*	EXPORT fn_incdate(ARG(ISC_QUAD*, date), ARG(long*, d), ARG(long*, m), ARG(long*, y))
ARGLIST(ISC_QUAD* ddate)
ARGLIST(long *d)
ARGLIST(long *m)
ARGLIST(long *y)
{
	ISC_QUAD *bufquad = (ISC_QUAD *) MALLOC (sizeof(ISC_QUAD));
	struct tm tm1;
	div_t x;
	long dpm, dd, mm, yy;

	isc_decode_date(date, &tm1);
	x = div(tm1.tm_mon + *m, 12);
	yy = tm1.tm_year + IB_START_YEAR + *y + x.quot;
	if (x.rem < 0)
	{
		mm = 12 + x.rem + 1;
		yy--;
	} else
		mm = x.rem + 1;

	dpm = fn_daypermonth(&mm, &yy);
	dd = (tm1.tm_mday <= dpm) ? tm1.tm_mday + *d : dpm + *d;
	if (dd >= 0)
	{
		while (dd > dpm)
		{
			if (mm < 12)
				mm++;
			else
			{
				yy++;
				mm = 1;
			}
			dd -= dpm;
			dpm = fn_daypermonth(&mm, &yy);
		}
	} else
	{
		mm--;
		dpm = fn_daypermonth(&yy, &mm);
		while (dpm + dd < 0)
		{
			if (mm > 1)
				mm--;
			else
			{
				yy--;
				mm = 12;
			}
			dd += dpm;
			dpm = fn_daypermonth(&mm, &yy);
		}
	}
	tm1.tm_hour = 0;
	tm1.tm_min = 0;
	tm1.tm_sec = 0;
	tm1.tm_mday = dd;
	tm1.tm_mon = mm - 1;
	tm1.tm_year = yy - IB_START_YEAR;
	isc_encode_date(&tm1, bufquad);

	return bufquad;
}

