/*====================================================================
		rmath.c

		rFunc InterBase UDF library.
		Math functions.

		Copyright 1998-2001 Polaris Software
		http://polesoft.da.ru
		mailto: rFunc@mail.ru

		Version 2.0.1.2
====================================================================== */

#include <stdlib.h>
#include <math.h>

#include "rfunc.h"
#include "rmath.h"

double EXPORT fn_abs(ARG(double*, x))
ARGLIST(double *x)
{
	return (*x < 0.0) ? -*x : *x;
}

double EXPORT fn_maxnum(ARG(double*, x), ARG(double*, y))
ARGLIST(double *x)
ARGLIST(double *y)
{
	return (*x > *y) ? *x : *y;
}

double EXPORT fn_minnum(ARG(double*, x), ARG(double*, y))
ARGLIST(double *x)
ARGLIST(double *y)
{
	return (*x < *y) ? *x : *y;
}

double EXPORT fn_ceil(ARG(double*, x))
ARGLIST(double *x)
{
	return ceil(*x);
}

double EXPORT fn_floor(ARG(double*, x))
ARGLIST(double *x)
{
	return floor(*x);
}

double EXPORT fn_power(ARG(double*, x), ARG(double*, y))
ARGLIST(double *x)
ARGLIST(double *y)
{
	return pow(*x,*y);
}

double EXPORT fn_round(ARG(double*, x), ARG(long*, n))
ARGLIST(double *x)
ARGLIST(long *n)
{
	 return *x>=0 ? floor((*x*pow(10,*n)+0.500000001))*pow(10,*n*(-1)) : ceil((*x*pow(10,*n)-0.500000001))*pow(10,*n*(-1));
}

double EXPORT fn_softround(ARG(double*, x), ARG(long*, n))
ARGLIST(double *x)
ARGLIST(long *n)
{
	 return (*x*pow(10,*n))>=1 ? fn_round(x, n) : *x;
}

long EXPORT fn_div(long* q, long* r)
ARGLIST(long *q)
ARGLIST(long *r)
{
	div_t x;
	x = div(*q, *r);
	return x.quot;
}

long EXPORT fn_mod(long* q, long* r)
ARGLIST(long *q)
ARGLIST(long *r)
{
	div_t x;
	x = div(*q, *r);
	return x.rem;
}

double EXPORT fn_z(ARG(double*, a))
ARGLIST(double *a)
{ return *a; }

double EXPORT fn_dividezero(ARG(double*, a), ARG(double*, b), ARG(double*, def))
ARGLIST(double *a)
ARGLIST(double *b)
ARGLIST(double *def)
{ return (*b) ? (*a / *b) : *def; }

long EXPORT fn_initRandom(ARG(long *, num))
ARGLIST(long *num)
{
#ifdef __WIN32__
	randomize();
#else
	srandom(*num);
#endif
	return 1;
}

long EXPORT fn_getRandom(ARG(long *, num))
ARGLIST(long *num)
{
#ifdef __WIN32__
	return random(*num);
#else
	div_t x;
	x = div(random(), *num);
	return x.rem;
#endif

}

long EXPORT fn_getBit(ARG(long *, num), ARG(long *, shift))
ARGLIST(long *num)
ARGLIST(long *shift)
{
	if ((*shift<0) || (*shift>31))
		return -1;
	if(((1<<*shift)&(*num))==0)
		return 0;
	else
		return 1;
}

long EXPORT fn_setBit(ARG(long *, num), ARG(long *, shift), ARG(long *, val))
ARGLIST(long *num)
ARGLIST(long *shift)
ARGLIST(long *val)
{
	if((*shift<0) || (*shift>31) || (*val<0) || (*val>1))
		return *num;
	else
	{
		if(*val==1)
			return (*num) | (1<<*shift);
		else
			return (*num) & (~(1<<*shift));
	}
}

long EXPORT fn_bitAnd(ARG(long *, a), ARG(long *, b))
ARGLIST(long *a)
ARGLIST(long *b)
{
	return (*a & *b);
}

long EXPORT fn_bitOr(ARG(long *, a), ARG(long *, b))
ARGLIST(long *a)
ARGLIST(long *b)
{
	return (*a | *b);
}

long EXPORT fn_bitXor(ARG(long *, a), ARG(long *, b))
ARGLIST(long *a)
ARGLIST(long *b)
{
	return (*a ^ *b);
}

long EXPORT fn_bitNot(ARG(long *, num))
ARGLIST(long *num)
{
	return ~(*num);
}

